"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("mz/fs");
const shelljs_1 = require("shelljs");
const path_1 = require("path");
const got_1 = require("got");
const pkgDir = require("pkg-dir");
const debug_1 = require("debug");
const debug = debug_1.default('typedi');
const REGISTRY_URL = 'https://registry.npmjs.org';
// some packages are exceptions - they ship with types, but those are stubs for some reason
// if a package is in this set, then @types/x will always be installed
const EXCEPTION_PACKAGES = new Set(['jest']);
exports.formatPackageMessage = (message, packages) => {
    return packages.length
        ? `${message}:\n${packages.map(p => `  * ${p}`).join('\n')}`
        : '';
};
const parseOpts = (packageManager) => {
    const command = {
        npm: 'npm i',
        yarn: 'yarn add',
        pnpm: 'pnpm i'
    }[packageManager];
    return {
        command,
        devFlag: '-D',
        exactFlag: '-E'
    };
};
exports.installWithTool = (modules, { packageManager = 'npm', dev = false, exact = false } = {}) => {
    if (!modules.length) {
        return Promise.resolve();
    }
    const { command, devFlag, exactFlag } = parseOpts(packageManager);
    return new Promise((res, rej) => shelljs_1.exec([command, dev ? devFlag : '', exact ? exactFlag : '', ...modules].join(' '), 
    // yarn works when silent, but npm doesn't
    { async: true, silent: true }, (code, stdout, stderr) => {
        if (code) {
            rej(stderr);
        }
        res(stdout);
    }));
};
/**
 * gets the `@types/name` registry info
 */
exports.getTypingInfo = async (name) => {
    const url = `${REGISTRY_URL}/@${encodeURIComponent(`types/${name}`)}`;
    const response = await got_1.default(url, { throwHttpErrors: false });
    if (response.statusCode === 404) {
        return null;
    }
    else if (response.statusCode >= 400) {
        throw new Error(`trouble reading from registry: ${response.body}`);
    }
    return name;
};
/**
 * returns `null` for functions that have type info
 * returns the module name if types are missing
 */
exports.missingTypes = async (m) => {
    debug('looking at', m);
    if (EXCEPTION_PACKAGES.has(m)) {
        debug(m, 'is an exception, returning');
        return m;
    }
    const pkgRoot = await pkgDir();
    const installDir = path_1.resolve(`${pkgRoot || '.'}/node_modules/${m}`);
    try {
        const pkg = require(`${installDir}/package.json`);
        // if the file exists at root, it doesn't need to be specified in pkg
        let orphanIndex = false;
        try {
            await fs_1.access(`${installDir}/index.d.ts`);
            orphanIndex = true;
        }
        catch (_a) { }
        if (pkg.typings || pkg.types || orphanIndex) {
            debug(m, 'has native types');
            return null;
        }
        else {
            debug(m, 'is missing types');
            return m;
        }
    }
    catch (e) {
        console.error('problem reading', m, '-', e);
        return null;
    }
};
